package com.biganto.visual.roompark.presentation.screen.estate

import android.net.Uri
import android.os.Bundle
import android.view.View
import android.webkit.WebView
import androidx.core.os.bundleOf
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.google.android.material.tabs.TabLayout
import com.google.android.material.textview.MaterialTextView
import com.jakewharton.rxbinding3.material.selections
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

const val SELECTED_ESTATE_ID_KEY = "SELECTED_ESTATE_INDEX"


class EstateScreenController  :
    BigantoBaseController<EstateScreenViewState
        , EstateScreen
        , EstateScreenPresenter>
    , EstateScreen {


    override fun planTypesTabSelected(): Observable<Int> =
        planTypesTabLayout.selections()
            .debounce (200L, TimeUnit.MILLISECONDS)
            .map { it.position }
            .observeOn(AndroidSchedulers.mainThread())


    constructor(args: Bundle):super(args)

    constructor(id: Int) : super(bundleOf(SELECTED_ESTATE_ID_KEY to id))

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: EstateScreenPresenter

    @BindView(R.id.planTypesTabs)
    lateinit var planTypesTabLayout: TabLayout

    @BindView(R.id.flat_plan_webview)
    lateinit var planWebView: WebView


    private fun setToolbar(){
        toolBar.showAll()
        toolBar.appBar.setExpanded(false,false)
        toolBar.collapsingToolbarLayout.title = "ИЗБРАННОЕ"
        toolBar.appBar.liftOnScrollTargetViewId = R.id.favorites_cards_recycler_view
        toolBar.appBar.setLiftable(true)
        toolBar.appBarScrollable(false)

//        planTypesTabLayout.
    }

    private fun bindRecycler() {

    }

    override fun onViewBound(v: View) {
        setToolbar()
        bindRecycler()
    }

    override fun render(viewState: EstateScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is EstateScreenViewState.Idle -> render(viewState)
            is EstateScreenViewState.LoadEstate -> render(viewState)
            is EstateScreenViewState.LoadPlanTypes -> render(viewState)
            is EstateScreenViewState.LoadPlan -> render(viewState)
            is EstateScreenViewState.SomeError -> render(viewState)
        }
    }

    private fun render(viewState: EstateScreenViewState.Idle){

    }

    private fun render(viewState: EstateScreenViewState.SomeError) =
        showError(viewState.exception)


    private fun render(viewState: EstateScreenViewState.LoadEstate) {
    }


    private fun render(viewState: EstateScreenViewState.LoadPlanTypes) {
        planTypesTabLayout.removeAllTabs()
        var i = 1

        viewState.types.forEach {
            try {

                val tab = planTypesTabLayout.newTab()
                    .setCustomView(R.layout.select_text_tab).setTag(it.planId)
                (tab.customView as MaterialTextView).text = "Вариант $i"
                planTypesTabLayout.addTab(tab)
                i++
            } catch (e: Exception) {
                Timber.e(e)
            }
        }
    }

    private fun render(viewState: EstateScreenViewState.LoadPlan) {
        planWebView.settings.javaScriptEnabled = true
        planWebView.clearCache(true)
        val uri = "file:///${Uri.parse(viewState.planBody).path}"
        planWebView.loadUrl(uri)
    }

        private fun getComponent() = DaggerEstateScreenComponent.factory()
        .create(RoomParkApplication.component,activity as RoomParkMainActivity
            ,args.getInt(SELECTED_ESTATE_ID_KEY))
        .inject(this)

    override fun getLayoutId(): Int = R.layout.flat_full_card_screen


}
