package com.biganto.visual.roompark.presentation.screen.estate

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.EstateInteractor
import com.biganto.visual.roompark.domain.model.PlanPresetModel
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject
import javax.inject.Named

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class EstateScreenPresenter @Inject constructor(
    private val interactor: EstateInteractor,
    @Named(SELECTED_ESTATE_ID_KEY) private val estateId:Int
)
    : BigantoBasePresenter<EstateScreen, EstateScreenViewState>() {

    private var planList:List<PlanPresetModel>? = null

    override fun defaultErrorViewStateHandler() =
        {e: ExceptionString -> EstateScreenViewState.SomeError(e) }

    override fun bindIntents() {

        val prefetchCards = interactor.getEstate(estateId)
            .map { EstateScreenViewState.LoadEstate(it) }
            //.map { EstateScreenViewState.FavoriteEstatesLoaded(it) }

        val fetchPlans = interactor.getPlanTypes(estateId)
            .doOnNext { planList = it.toList() }
            .map { EstateScreenViewState.LoadPlanTypes(it) }

        val fetchPlan = intent(EstateScreen::planTypesTabSelected)
            .map { plan ->planList?.first{it.planId == plan} }
            .flatMap { interactor.getPlan(it)
            }
            .map { EstateScreenViewState.LoadPlan(it) }

        val state = restoreStateObservable
            .mergeWith(prefetchCards)
            .mergeWith(fetchPlans)
            .mergeWith(fetchPlan)
            .doOnError{ Timber.e(it)}
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(EstateScreenViewState::class.java), EstateScreen::render)
    }

}